@@ -2,11 +2,9 @@
 import argparse
 import multiprocessing
 import pathlib
-import signal
 import time
 import traceback
 
-import colorama
 import filelock
 
 import sky
@@ -32,8 +30,8 @@ class SpotController:
     def __init__(self, job_id: int, task_yaml: str,
                  retry_until_up: bool) -> None:
         self._job_id = job_id
-        self._task_name = pathlib.Path(task_yaml).stem
         self._task = sky.Task.from_yaml(task_yaml)
+        self._task_name = self._task.name
 
         self._retry_until_up = retry_until_up
         # TODO(zhwu): this assumes the specific backend.
@@ -60,7 +58,27 @@ class SpotController:
             self._cluster_name, self._backend, self._task, retry_until_up)
 
     def _run(self):
-        """Busy loop monitoring spot cluster status and handling recovery."""
+        """Busy loop monitoring spot cluster status and handling recovery.
+
+        Raises:
+            exceptions.ProvisionPrechecksError: This will be raised when the
+                underlying `sky.launch` fails due to precheck errors only.
+                I.e., none of the failover exceptions, if
+                any, is due to resources unavailability. This exception
+                includes the following cases:
+                1. The optimizer cannot find a feasible solution.
+                2. Precheck errors: invalid cluster name, failure in getting
+                cloud user identity, or unsupported feature.
+            exceptions.SpotJobReachedMaxRetryError: This will be raised when
+                all prechecks passed but the maximum number of retries is
+                reached for `sky.launch`. The failure of `sky.launch` can be
+                due to:
+                1. Any of the underlying failover exceptions is due to resources
+                unavailability.
+                2. The cluster is preempted before the job is submitted.
+                3. Any unexpected error happens during the `sky.launch`.
+        Other exceptions may be raised depending on the backend.
+        """
         logger.info(f'Started monitoring spot task {self._task_name} '
                     f'(id: {self._job_id})')
         spot_state.set_starting(self._job_id)
@@ -134,11 +152,16 @@ class SpotController:
                                             None,
                                             spot_job_id=self._job_id)
                     logger.info(f'\n== End of logs (ID: {self._job_id}) ==')
-                    status_to_set = spot_state.SpotStatus.FAILED
+                    spot_status_to_set = spot_state.SpotStatus.FAILED
                     if job_status == job_lib.JobStatus.FAILED_SETUP:
-                        status_to_set = spot_state.SpotStatus.FAILED_SETUP
+                        spot_status_to_set = spot_state.SpotStatus.FAILED_SETUP
+                    failure_reason = (
+                        'To see the details, run: '
+                        f'sky spot logs --controller {self._job_id}')
+
                     spot_state.set_failed(self._job_id,
-                                          failure_type=status_to_set,
+                                          failure_type=spot_status_to_set,
+                                          failure_reason=failure_reason,
                                           end_time=end_time)
                     break
                 # Although the cluster is healthy, we fail to access the
@@ -157,7 +180,7 @@ class SpotController:
                     # Some spot resource (e.g., Spot TPU VM) may need to be
                     # cleaned up after preemption.
                     logger.info('Cleaning up the preempted spot cluster...')
-                    self._strategy_executor.terminate_cluster()
+                    recovery_strategy.terminate_cluster(self._cluster_name)
 
             # Try to recover the spot jobs, when the cluster is preempted
             # or the job status is failed to be fetched.
@@ -170,47 +193,40 @@ class SpotController:
         """Run controller logic and handle exceptions."""
         try:
             self._run()
-        except KeyboardInterrupt:
-            # Kill the children processes launched by log_lib.run_with_log.
-            subprocess_utils.kill_children_processes()
-            spot_state.set_cancelled(self._job_id)
-        except exceptions.ResourcesUnavailableError as e:
-            logger.error(f'{common_utils.class_fullname(e.__class__)}: '
-                         f'{colorama.Fore.RED}{e}{colorama.Style.RESET_ALL}')
+        except exceptions.ProvisionPrechecksError as e:
+            # Please refer to the docstring of self._run for the cases when
+            # this exception can occur.
+            failure_reason = ('; '.join(
+                common_utils.format_exception(reason, use_bracket=True)
+                for reason in e.reasons))
+            logger.error(failure_reason)
             spot_state.set_failed(
                 self._job_id,
-                failure_type=spot_state.SpotStatus.FAILED_NO_RESOURCE)
+                failure_type=spot_state.SpotStatus.FAILED_PRECHECKS,
+                failure_reason=failure_reason)
+        except exceptions.SpotJobReachedMaxRetriesError as e:
+            # Please refer to the docstring of self._run for the cases when
+            # this exception can occur.
+            logger.error(common_utils.format_exception(e))
+            # The spot job should be marked as FAILED_NO_RESOURCE, as the
+            # spot job may be able to launch next time.
+            spot_state.set_failed(
+                self._job_id,
+                failure_type=spot_state.SpotStatus.FAILED_NO_RESOURCE,
+                failure_reason=common_utils.format_exception(e))
         except (Exception, SystemExit) as e:  # pylint: disable=broad-except
             logger.error(traceback.format_exc())
-            logger.error('Unexpected error occurred: '
-                         f'{common_utils.format_exception(e)}')
-        finally:
-            self._strategy_executor.terminate_cluster()
-            job_status = spot_state.get_status(self._job_id)
-            # The job can be non-terminal if the controller exited abnormally,
-            # e.g. failed to launch cluster after reaching the MAX_RETRY.
-            if not job_status.is_terminal():
-                logger.info(f'Previous spot job status: {job_status.value}')
-                spot_state.set_failed(
-                    self._job_id,
-                    failure_type=spot_state.SpotStatus.FAILED_CONTROLLER)
-
-            # Clean up Storages with persistent=False.
-            self._backend.teardown_ephemeral_storage(self._task)
+            msg = ('Unexpected error occurred: '
+                   f'{common_utils.format_exception(e, use_bracket=True)}')
+            logger.error(msg)
+            spot_state.set_failed(
+                self._job_id,
+                failure_type=spot_state.SpotStatus.FAILED_CONTROLLER,
+                failure_reason=msg)
 
 
 def _run_controller(job_id: int, task_yaml: str, retry_until_up: bool):
     """Runs the controller in a remote process for interruption."""
-
-    # Override the SIGTERM handler to gracefully terminate the controller.
-    def handle_interupt(signum, frame):
-        """Handle the interrupt signal."""
-        # Need to raise KeyboardInterrupt to avoid the exception being caught by
-        # the strategy executor.
-        raise KeyboardInterrupt()
-
-    signal.signal(signal.SIGTERM, handle_interupt)
-
     # The controller needs to be instantiated in the remote process, since
     # the controller is not serializable.
     spot_controller = SpotController(job_id, task_yaml, retry_until_up)
@@ -247,11 +263,31 @@ def _handle_signal(job_id):
         f'User sent {user_signal.value} signal.')
 
 
+def _cleanup(job_id: int, task_yaml: str):
+    # NOTE: The code to get cluster name is same as what we did in the spot
+    # controller, we should keep it in sync with SpotController.__init__()
+    task = sky.Task.from_yaml(task_yaml)
+    task_name = task.name
+    cluster_name = spot_utils.generate_spot_cluster_name(task_name, job_id)
+    recovery_strategy.terminate_cluster(cluster_name)
+    # Clean up Storages with persistent=False.
+    # TODO(zhwu): this assumes the specific backend.
+    backend = cloud_vm_ray_backend.CloudVmRayBackend()
+    backend.teardown_ephemeral_storage(task)
+
+
 def start(job_id, task_yaml, retry_until_up):
     """Start the controller."""
     controller_process = None
+    cancelling = False
     try:
         _handle_signal(job_id)
+        # TODO(suquark): In theory, we should make controller process a
+        #  daemon process so it will be killed after this process exits,
+        #  however daemon process cannot launch subprocesses, explained here:
+        #  https://docs.python.org/3/library/multiprocessing.html#multiprocessing.Process.daemon  # pylint: disable=line-too-long
+        #  So we can only enable daemon after we no longer need to
+        #  start daemon processes like Ray.
         controller_process = multiprocessing.Process(target=_run_controller,
                                                      args=(job_id, task_yaml,
                                                            retry_until_up))
@@ -261,15 +297,47 @@ def start(job_id, task_yaml, retry_until_up):
             time.sleep(1)
     except exceptions.SpotUserCancelledError:
         logger.info(f'Cancelling spot job {job_id}...')
+        spot_state.set_cancelling(job_id)
+        cancelling = True
+    finally:
         if controller_process is not None:
-            logger.info('Sending SIGTERM to controller process '
-                        f'{controller_process.pid}')
-            # This will raise KeyboardInterrupt in the task.
-            # Using SIGTERM instead of SIGINT, as the SIGINT is weirdly ignored
-            # by the controller process when it is started inside a ray job.
-            controller_process.terminate()
-    if controller_process is not None:
-        controller_process.join()
+            logger.info(f'Killing controller process {controller_process.pid}')
+            # NOTE: it is ok to kill or join a killed process.
+            # Kill the controller process first; if its child process is
+            # killed first, then the controller process will raise errors.
+            # Kill any possible remaining children processes recursively.
+            subprocess_utils.kill_children_processes(controller_process.pid,
+                                                     force=True)
+            controller_process.join()
+            logger.info(f'Controller process {controller_process.pid} killed.')
+
+        logger.info(f'Cleaning up spot clusters of job {job_id}.')
+        # NOTE: Originally, we send an interruption signal to the controller
+        # process and the controller process handles cleanup. However, we
+        # figure out the behavior differs from cloud to cloud
+        # (e.g., GCP ignores 'SIGINT'). A possible explanation is
+        # https://unix.stackexchange.com/questions/356408/strange-problem-with-trap-and-sigint
+        # But anyway, a clean solution is killing the controller process
+        # directly, and then cleanup the cluster state.
+        _cleanup(job_id, task_yaml=task_yaml)
+        logger.info(f'Spot clusters of job {job_id} has been taken down.')
+
+        if cancelling:
+            spot_state.set_cancelled(job_id)
+
+        # We should check job status after 'set_cancelled', otherwise
+        # the job status is not terminal.
+        job_status = spot_state.get_status(job_id)
+        # The job can be non-terminal if the controller exited abnormally,
+        # e.g. failed to launch cluster after reaching the MAX_RETRY.
+        assert job_status is not None
+        if not job_status.is_terminal():
+            logger.info(f'Previous spot job status: {job_status.value}')
+            spot_state.set_failed(
+                job_id,
+                failure_type=spot_state.SpotStatus.FAILED_CONTROLLER,
+                failure_reason=('Unexpected error occurred. For details, '
+                                f'run: sky spot logs --controller {job_id}'))
 
 
 if __name__ == '__main__':
@@ -283,7 +351,9 @@ if __name__ == '__main__':
                         help='Retry until the spot cluster is up.')
     parser.add_argument('task_yaml',
                         type=str,
-                        help='The path to the user spot task yaml file. '
-                        'The file name is the spot task name.')
+                        help='The path to the user spot task yaml file.')
     args = parser.parse_args()
+    # We start process with 'spawn', because 'fork' could result in weird
+    # behaviors; 'spawn' is also cross-platform.
+    multiprocessing.set_start_method('spawn', force=True)
     start(args.job_id, args.task_yaml, args.retry_until_up)
