@@ -1,5 +1,6 @@
 """The strategy to handle launching/recovery/termination of spot clusters."""
 import time
+import traceback
 import typing
 from typing import Optional, Tuple
 
@@ -7,6 +8,7 @@ import sky
 from sky import exceptions
 from sky import global_user_state
 from sky import sky_logging
+from sky import backends
 from sky.backends import backend_utils
 from sky.skylet import job_lib
 from sky.spot import spot_utils
@@ -15,12 +17,11 @@ from sky.utils import common_utils
 from sky.utils import ux_utils
 
 if typing.TYPE_CHECKING:
-    from sky import backends
     from sky import task as task_lib
 
 logger = sky_logging.init_logger(__name__)
 
-SPOT_STRATEGIES = dict()
+SPOT_STRATEGIES = {}
 SPOT_DEFAULT_STRATEGY = None
 
 # Waiting time for job from INIT/PENDING to RUNNING
@@ -28,6 +29,28 @@ SPOT_DEFAULT_STRATEGY = None
 MAX_JOB_CHECKING_RETRY = 10
 
 
+def terminate_cluster(cluster_name: str, max_retry: int = 3) -> None:
+    """Terminate the spot cluster."""
+    retry_cnt = 0
+    while True:
+        try:
+            usage_lib.messages.usage.set_internal()
+            sky.down(cluster_name)
+            return
+        except ValueError:
+            # The cluster is already down.
+            return
+        except Exception as e:  # pylint: disable=broad-except
+            retry_cnt += 1
+            if retry_cnt >= max_retry:
+                raise RuntimeError('Failed to terminate the spot cluster '
+                                   f'{cluster_name}.') from e
+            logger.error('Failed to terminate the spot cluster '
+                         f'{cluster_name}. Retrying.'
+                         f'Details: {common_utils.format_exception(e)}')
+            logger.error(f'  Traceback: {traceback.format_exc()}')
+
+
 class StrategyExecutor:
     """Handle each launching, recovery and termination of the spot clusters."""
 
@@ -43,6 +66,8 @@ class StrategyExecutor:
             task: The task to execute.
             retry_until_up: Whether to retry until the cluster is up.
         """
+        assert isinstance(backend, backends.CloudVmRayBackend), (
+            'Only CloudVMRayBackend is supported.')
         self.dag = sky.Dag()
         self.dag.add(task)
         self.cluster_name = cluster_name
@@ -81,6 +106,8 @@ class StrategyExecutor:
         status, after calling.
 
         Returns: The job's submit timestamp, or None if failed.
+
+        Raises: Please refer to the docstring of self._launch().
         """
         if self.retry_until_up:
             return self._launch(max_retry=None)
@@ -96,32 +123,14 @@ class StrategyExecutor:
         """
         raise NotImplementedError
 
-    def terminate_cluster(self, max_retry: int = 3) -> None:
-        """Terminate the spot cluster."""
-        retry_cnt = 0
-        while True:
-            try:
-                handle = global_user_state.get_handle_from_cluster_name(
-                    self.cluster_name)
-                if handle is None:
-                    return
-                self.backend.teardown(handle, terminate=True)
-                return
-            except Exception as e:  # pylint: disable=broad-except
-                retry_cnt += 1
-                if retry_cnt >= max_retry:
-                    raise RuntimeError('Failed to terminate the spot cluster '
-                                       f'{self.cluster_name}.') from e
-                logger.error('Failed to terminate the spot cluster '
-                             f'{self.cluster_name}. Retrying.')
-
     def _try_cancel_all_jobs(self):
         handle = global_user_state.get_handle_from_cluster_name(
             self.cluster_name)
         if handle is None:
             return
         try:
-            self.backend.cancel_jobs(handle, jobs=None)
+            usage_lib.messages.usage.set_internal()
+            sky.cancel(cluster_name=self.cluster_name, all=True)
         except Exception as e:  # pylint: disable=broad-except
             # Ignore the failure as the cluster can be totally stopped, and the
             # job canceling can get connection error.
@@ -129,6 +138,70 @@ class StrategyExecutor:
                         'the spot cluster is likely completely stopped.'
                         f'\n  Detailed exception: {e}')
 
+    def _wait_until_job_starts_on_cluster(self) -> Optional[float]:
+        """Wait for MAX_JOB_CHECKING_RETRY times until job starts on the cluster
+
+        Returns:
+            The timestamp of when the job is submitted, or None if failed to
+            submit.
+        """
+        status = None
+        job_checking_retry_cnt = 0
+        while job_checking_retry_cnt < MAX_JOB_CHECKING_RETRY:
+            # Avoid the infinite loop, if any bug happens.
+            job_checking_retry_cnt += 1
+            try:
+                cluster_status, _ = (
+                    backend_utils.refresh_cluster_status_handle(
+                        self.cluster_name, force_refresh=True))
+            except Exception as e:  # pylint: disable=broad-except
+                # If any unexpected error happens, retry the job checking
+                # loop.
+                # TODO(zhwu): log the unexpected error to usage collection
+                # for future debugging.
+                logger.info(f'Unexpected exception: {e}\nFailed to get the '
+                            'refresh the cluster status. Retrying.')
+                continue
+            if cluster_status != global_user_state.ClusterStatus.UP:
+                # The cluster can be preempted before the job is
+                # launched.
+                # Break to let the retry launch kick in.
+                logger.info('The cluster is preempted before the job '
+                            'is submitted.')
+                # TODO(zhwu): we should recover the preemption with the
+                # recovery strategy instead of the current while loop.
+                break
+
+            try:
+                status = spot_utils.get_job_status(self.backend,
+                                                   self.cluster_name)
+            except Exception as e:  # pylint: disable=broad-except
+                # If any unexpected error happens, retry the job checking
+                # loop.
+                # Note: the CommandError is already handled in the
+                # get_job_status, so it should not happen here.
+                # TODO(zhwu): log the unexpected error to usage collection
+                # for future debugging.
+                logger.info(f'Unexpected exception: {e}\nFailed to get the '
+                            'job status. Retrying.')
+                continue
+
+            # Check the job status until it is not in initialized status
+            if status is not None and status > job_lib.JobStatus.INIT:
+                try:
+                    job_submitted_at = spot_utils.get_job_timestamp(
+                        self.backend, self.cluster_name, get_end_time=False)
+                    return job_submitted_at
+                except Exception as e:  # pylint: disable=broad-except
+                    # If we failed to get the job timestamp, we will retry
+                    # job checking loop.
+                    logger.info(f'Unexpected Exception: {e}\nFailed to get '
+                                'the job start timestamp. Retrying.')
+                    continue
+            # Wait for the job to be started
+            time.sleep(spot_utils.JOB_STARTED_STATUS_CHECK_GAP_SECONDS)
+        return None
+
     def _launch(self, max_retry=3, raise_on_failure=True) -> Optional[float]:
         """Implementation of launch().
 
@@ -143,14 +216,32 @@ class StrategyExecutor:
             The job's submit timestamp, or None if failed to submit the job
             (either provisioning fails or any error happens in job submission)
             and raise_on_failure is False.
+
+        Raises:
+            non-exhaustive list of exceptions:
+            exceptions.ProvisionPrechecksError: This will be raised when the
+                underlying `sky.launch` fails due to precheck errors only.
+                I.e., none of the failover exceptions, if
+                any, is due to resources unavailability. This exception
+                includes the following cases:
+                1. The optimizer cannot find a feasible solution.
+                2. Precheck errors: invalid cluster name, failure in getting
+                cloud user identity, or unsupported feature.
+            exceptions.SpotJobReachedMaxRetryError: This will be raised when
+                all prechecks passed but the maximum number of retries is
+                reached for `sky.launch`. The failure of `sky.launch` can be
+                due to:
+                1. Any of the underlying failover exceptions is due to resources
+                unavailability.
+                2. The cluster is preempted before the job is submitted.
+                3. Any unexpected error happens during the `sky.launch`.
+        Other exceptions may be raised depending on the backend.
         """
         # TODO(zhwu): handle the failure during `preparing sky runtime`.
         retry_cnt = 0
         backoff = common_utils.Backoff(self.RETRY_INIT_GAP_SECONDS)
         while True:
             retry_cnt += 1
-            retry_launch = False
-            exception = None
             try:
                 usage_lib.messages.usage.set_internal()
                 # Detach setup, so that the setup failure can be detected
@@ -162,95 +253,68 @@ class StrategyExecutor:
                            _is_launched_by_spot_controller=True)
                 logger.info('Spot cluster launched.')
             except exceptions.InvalidClusterNameError as e:
-                # The cluster name is too long.
-                raise exceptions.ResourcesUnavailableError(str(e)) from e
+                logger.error('Failure happened before provisioning. '
+                             f'{common_utils.format_exception(e)}')
+                if raise_on_failure:
+                    raise exceptions.ProvisionPrechecksError(reasons=[e])
+                return None
+            except exceptions.ResourcesUnavailableError as e:
+                # This is raised when the launch fails due to prechecks or
+                # after failing over through all the candidates.
+                # Please refer to the docstring of `sky.launch` for more
+                # details of how the exception will be structured.
+                if not any(
+                        isinstance(err, exceptions.ResourcesUnavailableError)
+                        for err in e.failover_history):
+                    # _launch() (this function) should fail/exit directly, if
+                    # none of the failover reasons were because of resource
+                    # unavailability or no failover was attempted (the optimizer
+                    # cannot find feasible resources for requested resources),
+                    # i.e., e.failover_history is empty.
+                    # Failing directly avoids the infinite loop of retrying
+                    # the launch when, e.g., an invalid cluster name is used
+                    # and --retry-until-up is specified.
+                    reasons = (e.failover_history
+                               if e.failover_history else [e])
+                    reasons_str = '; '.join(
+                        common_utils.format_exception(err) for err in reasons)
+                    logger.error(
+                        'Failure happened before provisioning. Failover '
+                        f'reasons: {reasons_str}')
+                    if raise_on_failure:
+                        raise exceptions.ProvisionPrechecksError(
+                            reasons=reasons)
+                    return None
+                logger.info('Failed to launch the spot cluster with error: '
+                            f'{common_utils.format_exception(e)})')
             except Exception as e:  # pylint: disable=broad-except
                 # If the launch fails, it will be recovered by the following
                 # code.
                 logger.info('Failed to launch the spot cluster with error: '
                             f'{common_utils.format_exception(e)})')
-                import traceback  # pylint: disable=import-outside-toplevel
                 logger.info(f'  Traceback: {traceback.format_exc()}')
-                retry_launch = True
-                exception = e
+            else:  # No exception, the launch succeeds.
+                # At this point, a sky.launch() has succeeded. Cluster may be
+                # UP (no preemption since) or DOWN (newly preempted).
+                job_submitted_at = self._wait_until_job_starts_on_cluster()
+                if job_submitted_at is not None:
+                    return job_submitted_at
+                # The job fails to start on the spot cluster, retry the launch.
+                # TODO(zhwu): log the unexpected error to usage collection
+                # for future debugging.
+                logger.info(
+                    'Failed to successfully submit the job to the '
+                    'launched cluster, due to unexpected submission errors or '
+                    'the cluster being preempted during job submission.')
 
-            # At this point, a sky.launch() has succeeded. Cluster may be
-            # UP (no preemption since) or DOWN (newly preempted).
-            status = None
-            job_checking_retry_cnt = 0
-            while not retry_launch:
-                job_checking_retry_cnt += 1
-                if job_checking_retry_cnt >= MAX_JOB_CHECKING_RETRY:
-                    # Avoid the infinite loop, if any bug happens.
-                    retry_launch = True
-                    # TODO(zhwu): log the unexpected error to usage collection
-                    # for future debugging.
-                    logger.info(
-                        'Failed to get the job status, due to unexpected '
-                        'job submission error.')
-                    break
-
-                try:
-                    cluster_status, _ = (
-                        backend_utils.refresh_cluster_status_handle(
-                            self.cluster_name, force_refresh=True))
-                except Exception as e:  # pylint: disable=broad-except
-                    # If any unexpected error happens, retry the job checking
-                    # loop.
-                    # TODO(zhwu): log the unexpected error to usage collection
-                    # for future debugging.
-                    logger.info(f'Unexpected exception: {e}\nFailed to get the '
-                                'refresh the cluster status. Retrying.')
-                    continue
-                if cluster_status != global_user_state.ClusterStatus.UP:
-                    # The cluster can be preempted before the job is launched.
-                    # Break to let the retry launch kick in.
-                    logger.info('The cluster is preempted before the job '
-                                'is submitted.')
-                    # TODO(zhwu): we should recover the preemption with the
-                    # recovery strategy instead of the current while loop.
-                    retry_launch = True
-                    break
-
-                try:
-                    status = spot_utils.get_job_status(self.backend,
-                                                       self.cluster_name)
-                except Exception as e:  # pylint: disable=broad-except
-                    # If any unexpected error happens, retry the job checking
-                    # loop.
-                    # Note: the CommandError is already handled in the
-                    # get_job_status, so it should not happen here.
-                    # TODO(zhwu): log the unexpected error to usage collection
-                    # for future debugging.
-                    logger.info(f'Unexpected exception: {e}\nFailed to get the '
-                                'job status. Retrying.')
-                    continue
-
-                # Check the job status until it is not in initialized status
-                if status is not None and status > job_lib.JobStatus.INIT:
-                    try:
-                        job_submitted_at = spot_utils.get_job_timestamp(
-                            self.backend, self.cluster_name, get_end_time=False)
-                        return job_submitted_at
-                    except Exception as e:  # pylint: disable=broad-except
-                        # If we failed to get the job timestamp, we will retry
-                        # job checking loop.
-                        logger.info(f'Unexpected Exception: {e}\nFailed to get '
-                                    'the job start timestamp. Retrying.')
-                        continue
-                # Wait for the job to be started
-                time.sleep(spot_utils.JOB_STARTED_STATUS_CHECK_GAP_SECONDS)
-
-            assert retry_launch
-
-            self.terminate_cluster()
+            terminate_cluster(self.cluster_name)
             if max_retry is not None and retry_cnt >= max_retry:
                 # Retry forever if max_retry is None.
                 if raise_on_failure:
                     with ux_utils.print_exception_no_traceback():
-                        raise exceptions.ResourcesUnavailableError(
-                            'Failed to launch the spot cluster after '
-                            f'{max_retry} retries.') from exception
+                        raise exceptions.SpotJobReachedMaxRetriesError(
+                            'Resources unavailable: failed to launch the spot '
+                            f'cluster after {max_retry} retries.')
                 else:
                     return None
             gap_seconds = backoff.current_backoff()
@@ -322,7 +386,7 @@ class FailoverStrategyExecutor(StrategyExecutor, name='FAILOVER', default=True):
             logger.debug('Terminating unhealthy spot cluster and '
                          'reset cloud region.')
             self._launched_cloud_region = None
-            self.terminate_cluster()
+            terminate_cluster(self.cluster_name)
 
             # Step 3
             logger.debug('Relaunch the cluster  without constraining to prior '
