@@ -142,7 +142,7 @@ class Task:
         self.storage_mounts = {}
         self.storage_plans = {}
         self.setup = setup
-        self._envs = envs or dict()
+        self._envs = envs or {}
         self.workdir = workdir
         self.docker_image = (docker_image if docker_image else
                              'gpuci/miniforge-cuda:11.4-devel-ubuntu18.04')
@@ -274,7 +274,7 @@ class Task:
         fm_storages = []
         file_mounts = config.pop('file_mounts', None)
         if file_mounts is not None:
-            copy_mounts = dict()
+            copy_mounts = {}
             for dst_path, src in file_mounts.items():
                 # Check if it is str path
                 if isinstance(src, str):
@@ -353,7 +353,7 @@ class Task:
           ValueError: if various invalid inputs errors are detected.
         """
         if envs is None:
-            self._envs = dict()
+            self._envs = {}
             return self
         if isinstance(envs, (list, tuple)):
             keys = set(env[0] for env in envs)
@@ -471,8 +471,8 @@ class Task:
         """
         if self.time_estimator_func is None:
             raise NotImplementedError(
-                'Node [{}] does not have a cost model set; '
-                'call set_time_estimator() first'.format(self))
+                f'Node [{self}] does not have a cost model set; '
+                'call set_time_estimator() first')
         return self.time_estimator_func(resources)
 
     def set_file_mounts(self, file_mounts: Optional[Dict[str, str]]) -> 'Task':
@@ -717,8 +717,9 @@ class Task:
                 store_type = storage_plans[storage]
                 if store_type is storage_lib.StoreType.S3:
                     # TODO: allow for Storage mounting of different clouds
-                    if storage.source is not None and storage.source.startswith(
-                            's3://'):
+                    if storage.source is not None and not isinstance(
+                            storage.source,
+                            list) and storage.source.startswith('s3://'):
                         blob_path = storage.source
                     else:
                         blob_path = 's3://' + storage.name
@@ -726,7 +727,9 @@ class Task:
                         mnt_path: blob_path,
                     })
                 elif store_type is storage_lib.StoreType.GCS:
-                    if storage.source.startswith('gs://'):
+                    if storage.source is not None and not isinstance(
+                            storage.source,
+                            list) and storage.source.startswith('gs://'):
                         blob_path = storage.source
                     else:
                         blob_path = 'gs://' + storage.name
@@ -780,7 +783,7 @@ class Task:
 
         INTERNAL: this method is internal-facing.
         """
-        config = dict()
+        config = {}
 
         def add_if_not_none(key, value, no_empty: bool = False):
             if no_empty and not value:
@@ -809,7 +812,7 @@ class Task:
         add_if_not_none('run', self.run)
         add_if_not_none('envs', self.envs, no_empty=True)
 
-        add_if_not_none('file_mounts', dict())
+        add_if_not_none('file_mounts', {})
 
         if self.file_mounts is not None:
             config['file_mounts'].update(self.file_mounts)
