@@ -4,6 +4,7 @@ import click
 import colorama
 
 from sky import backends
+from sky import global_user_state
 from sky import spot
 from sky.backends import backend_utils
 from sky.utils import common_utils
@@ -11,6 +12,11 @@ from sky.utils import log_utils
 
 _COMMAND_TRUNC_LENGTH = 25
 
+# A record in global_user_state's 'clusters' table.
+_ClusterRecord = Dict[str, Any]
+# A record returned by core.cost_report(); see its docstr for all fields.
+_ClusterCostReportRecord = Dict[str, Any]
+
 
 def _truncate_long_string(s: str, max_length: int = 35) -> str:
     if len(s) <= max_length:
@@ -51,13 +57,13 @@ class StatusColumn:
         return val
 
 
-def show_status_table(cluster_records: List[Dict[str, Any]],
-                      show_all: bool,
-                      reserved_group_name: Optional[str] = None) -> int:
+def show_status_table(cluster_records: List[_ClusterRecord],
+                      show_all: bool) -> int:
     """Compute cluster table values and display.
 
     Returns:
-        Number of pending auto{stop,down} clusters.
+        Number of pending auto{stop,down} clusters that are not already
+        STOPPED.
     """
     # TODO(zhwu): Update the information for autostop clusters.
 
@@ -69,7 +75,7 @@ def show_status_table(cluster_records: List[Dict[str, Any]],
                      trunc_length=70 if not show_all else 0),
         StatusColumn('REGION', _get_region, show_by_default=False),
         StatusColumn('ZONE', _get_zone, show_by_default=False),
-        StatusColumn('STATUS', _get_status),
+        StatusColumn('STATUS', _get_status_colored),
         StatusColumn('AUTOSTOP', _get_autostop),
         StatusColumn('COMMAND',
                      _get_command,
@@ -82,37 +88,23 @@ def show_status_table(cluster_records: List[Dict[str, Any]],
             columns.append(status_column.name)
     cluster_table = log_utils.create_table(columns)
 
-    pending_autostop = 0
+    num_pending_autostop = 0
     for record in cluster_records:
         row = []
         for status_column in status_columns:
             if status_column.show_by_default or show_all:
                 row.append(status_column.calc(record))
         cluster_table.add_row(row)
-        pending_autostop += _is_pending_autostop(record)
+        num_pending_autostop += _is_pending_autostop(record)
 
     if cluster_records:
-        if reserved_group_name is not None:
-            autostop_minutes = spot.SPOT_CONTROLLER_IDLE_MINUTES_TO_AUTOSTOP
-            click.echo(f'\n{colorama.Fore.CYAN}{colorama.Style.BRIGHT}'
-                       f'{reserved_group_name}{colorama.Style.RESET_ALL}'
-                       f'{colorama.Style.DIM} (autostopped if idle for '
-                       f'{autostop_minutes}min)'
-                       f'{colorama.Style.RESET_ALL}')
-            reset = backend_utils.RESET_BOLD
-            click.echo('Use spot jobs CLI: '
-                       f'{colorama.Style.BRIGHT}sky spot --help{reset}')
-
-        else:
-            click.echo(f'{colorama.Fore.CYAN}{colorama.Style.BRIGHT}Clusters'
-                       f'{colorama.Style.RESET_ALL}')
         click.echo(cluster_table)
     else:
         click.echo('No existing clusters.')
-    return pending_autostop
+    return num_pending_autostop
 
 
-def show_cost_report_table(cluster_records: List[Dict[str, Any]],
+def show_cost_report_table(cluster_records: List[_ClusterCostReportRecord],
                            show_all: bool,
                            reserved_group_name: Optional[str] = None):
     """Compute cluster table values and display for cost report.
@@ -129,8 +121,15 @@ def show_cost_report_table(cluster_records: List[Dict[str, Any]],
         StatusColumn('RESOURCES',
                      _get_resources_for_cost_report,
                      trunc_length=70 if not show_all else 0),
-        StatusColumn('HOURLY_PRICE', _get_price, show_by_default=True),
-        StatusColumn('COST (est.)', get_cost_report, show_by_default=True),
+        StatusColumn('STATUS',
+                     _get_status_for_cost_report,
+                     show_by_default=True),
+        StatusColumn('HOURLY_PRICE',
+                     _get_price_for_cost_report,
+                     show_by_default=True),
+        StatusColumn('COST (est.)',
+                     _get_estimated_cost_for_cost_report,
+                     show_by_default=True),
     ]
 
     columns = []
@@ -192,7 +191,7 @@ def show_local_status_table(local_clusters: List[str]):
         config = common_utils.read_yaml(config_path)
         username = config['auth']['ssh_user']
 
-        if not isinstance(handle, backends.CloudVmRayBackend.ResourceHandle):
+        if not isinstance(handle, backends.CloudVmRayResourceHandle):
             raise ValueError(f'Unknown handle type {type(handle)} encountered.')
 
         if (handle.launched_nodes is not None and
@@ -257,27 +256,34 @@ def show_local_status_table(local_clusters: List[str]):
         click.echo(cluster_table)
 
 
-_get_name = (lambda cluster_status: cluster_status['name'])
-_get_launched = (lambda cluster_status: log_utils.readable_time_duration(
-    cluster_status['launched_at']))
+# Some of these lambdas are invoked on both _ClusterRecord and
+# _ClusterCostReportRecord, which is okay as we guarantee the queried fields
+# exist in those cases.
+_get_name = (lambda cluster_record: cluster_record['name'])
+_get_launched = (lambda cluster_record: log_utils.readable_time_duration(
+    cluster_record['launched_at']))
 _get_region = (
     lambda clusters_status: clusters_status['handle'].launched_resources.region)
-_get_command = (lambda cluster_status: cluster_status['last_use'])
-_get_duration = (lambda cluster_status: log_utils.readable_time_duration(
-    0, cluster_status['duration'], absolute=True))
+_get_command = (lambda cluster_record: cluster_record['last_use'])
+_get_duration = (lambda cluster_record: log_utils.readable_time_duration(
+    0, cluster_record['duration'], absolute=True))
 
 
-def _get_status(cluster_status):
-    status = cluster_status['status']
-    return status.colored_str()
+def _get_status(
+        cluster_record: _ClusterRecord) -> global_user_state.ClusterStatus:
+    return cluster_record['status']
 
 
-def _get_resources(cluster_status):
-    handle = cluster_status['handle']
+def _get_status_colored(cluster_record: _ClusterRecord) -> str:
+    return _get_status(cluster_record).colored_str()
+
+
+def _get_resources(cluster_record: _ClusterRecord) -> str:
+    handle = cluster_record['handle']
     resources_str = '<initializing>'
-    if isinstance(handle, backends.LocalDockerBackend.ResourceHandle):
+    if isinstance(handle, backends.LocalDockerResourceHandle):
         resources_str = 'docker'
-    elif isinstance(handle, backends.CloudVmRayBackend.ResourceHandle):
+    elif isinstance(handle, backends.CloudVmRayResourceHandle):
         if (handle.launched_nodes is not None and
                 handle.launched_resources is not None):
             launched_resource_str = str(handle.launched_resources)
@@ -288,9 +294,49 @@ def _get_resources(cluster_status):
     return resources_str
 
 
-def _get_resources_for_cost_report(cluster_status):
-    launched_nodes = cluster_status['num_nodes']
-    launched_resources = cluster_status['resources']
+def _get_zone(cluster_record: _ClusterRecord) -> str:
+    zone_str = cluster_record['handle'].launched_resources.zone
+    if zone_str is None:
+        zone_str = '-'
+    return zone_str
+
+
+def _get_autostop(cluster_record: _ClusterRecord) -> str:
+    autostop_str = ''
+    separtion = ''
+    if cluster_record['autostop'] >= 0:
+        # TODO(zhwu): check the status of the autostop cluster.
+        autostop_str = str(cluster_record['autostop']) + 'm'
+        separtion = ' '
+
+    if cluster_record['to_down']:
+        autostop_str += f'{separtion}(down)'
+    if autostop_str == '':
+        autostop_str = '-'
+    return autostop_str
+
+
+def _is_pending_autostop(cluster_record: _ClusterRecord) -> bool:
+    # autostop < 0 means nothing scheduled.
+    return cluster_record['autostop'] >= 0 and _get_status(
+        cluster_record) != global_user_state.ClusterStatus.STOPPED
+
+
+# ---- 'sky cost-report' helper functions below ----
+
+
+def _get_status_for_cost_report(
+        cluster_cost_report_record: _ClusterCostReportRecord) -> str:
+    status = cluster_cost_report_record['status']
+    if status is None:
+        return f'{colorama.Style.DIM}TERMINATED{colorama.Style.RESET_ALL}'
+    return status.colored_str()
+
+
+def _get_resources_for_cost_report(
+        cluster_cost_report_record: _ClusterCostReportRecord) -> str:
+    launched_nodes = cluster_cost_report_record['num_nodes']
+    launched_resources = cluster_cost_report_record['resources']
 
     launched_resource_str = str(launched_resources)
     resources_str = (f'{launched_nodes}x '
@@ -299,46 +345,21 @@ def _get_resources_for_cost_report(cluster_status):
     return resources_str
 
 
-def _get_zone(cluster_status):
-    zone_str = cluster_status['handle'].launched_resources.zone
-    if zone_str is None:
-        zone_str = '-'
-    return zone_str
-
-
-def _get_price(cluster_status):
-    launched_nodes = cluster_status['num_nodes']
-    launched_resources = cluster_status['resources']
+def _get_price_for_cost_report(
+        cluster_cost_report_record: _ClusterCostReportRecord) -> str:
+    launched_nodes = cluster_cost_report_record['num_nodes']
+    launched_resources = cluster_cost_report_record['resources']
 
     hourly_cost = (launched_resources.get_cost(3600) * launched_nodes)
     price_str = f'$ {hourly_cost:.3f}'
     return price_str
 
 
-def _get_autostop(cluster_status):
-    autostop_str = ''
-    separtion = ''
-    if cluster_status['autostop'] >= 0:
-        # TODO(zhwu): check the status of the autostop cluster.
-        autostop_str = str(cluster_status['autostop']) + 'm'
-        separtion = ' '
-
-    if cluster_status['to_down']:
-        autostop_str += f'{separtion}(down)'
-    if autostop_str == '':
-        autostop_str = '-'
-    return autostop_str
-
-
-def get_cost_report(cluster_status):
-    cost = cluster_status['total_cost']
+def _get_estimated_cost_for_cost_report(
+        cluster_cost_report_record: _ClusterCostReportRecord) -> str:
+    cost = cluster_cost_report_record['total_cost']
 
     if not cost:
         return '-'
 
     return f'${cost:.3f}'
-
-
-def _is_pending_autostop(cluster_status):
-    return _get_autostop(cluster_status) != '-' and _get_status(
-        cluster_status) != 'STOPPED'
