@@ -1,6 +1,5 @@
 """Utility functions for UX."""
 import contextlib
-import os
 import sys
 
 import rich.console as rich_console
@@ -33,12 +32,3 @@ def print_exception_no_traceback():
     sys.tracebacklimit = 0
     yield
     sys.tracebacklimit = original_tracelimit
-
-
-@contextlib.contextmanager
-def suppress_output():
-    """Suppress stdout and stderr."""
-    with open(os.devnull, 'w') as devnull:
-        with contextlib.redirect_stdout(devnull), contextlib.redirect_stderr(
-                devnull):
-            yield
